const fetchModule = require("fetch");
const Connectivity = require("connectivity");
const DataTransform = require("node-json-transform").DataTransform;
const FeedbackPlugin = require("nativescript-feedback");

const config = require("../shared/config");
const utils = require("../shared/utils");


function LMS() {

	this.feedback = new FeedbackPlugin.Feedback();

	this.playlistMap = {
		list : 'playlists_loop',
		item: {
			playlistId: "id",
			playlistName: "playlist",
		}
	};

	// adlu
	this.playlistTracksMap = {
		list : 'playlisttracks_loop',
		item: {
			playlistIndex:"playlist index",
			trackId:"id",
			trackTitle:"title",
			trackArtist:"artist",
			trackDuration:"duration",
			trackAlbum:"album",
			trackUrl:"url",
		}
	};


	this.artistMap = {
		list : 'artists_loop',
		item: {
			artistId: "id",
			artistName: "artist",
		}
	};

	this.albumMap = {
		list : 'albums_loop',
		item: {
			albumId: "id",
			albumName: "album",
			albumYear: "year",
			albumDisc: "disc",
			albumCompilation: "compilation",
			albumArtist: "artist",
			albumArtists: "artists",
			albumArtistId: "artist_id",
			albumArtistIds: "artist_ids"
		},
		operate: [
			{
				'run': function(val) { return String(val).split(',').map(Number)}, 'on': 'albumArtistIds'
			}
		]
	};

	this.trackMap = {
		list : 'titles_loop',
		item: {
			trackId:"id",
			trackTitle:"title",
			trackArtist:"artist",
			trackArtistId:"artist_id",
			trackArtistIds: "artist_ids",
			trackCoverId:"coverid",
			trackCompilation:"compilation",
			trackDuration:"duration",
			trackAlbumId:"album_id",
			trackFilesize:"filesize",
			trackGenre:"genre",
			trackGenres:"genres",
			trackDisc:"disc",
			trackArtworkTrackId:"artwork_track_id",
			trackComment:"comment",
			trackAlbum:"album",
			trackModificationTime:"modificationTime",
			trackGenreId:"genre_id",
			trackGenreIds:"genre_ids",
			trackAddedTime:"addedTime",
			trackDisccount:"disccount",
			trackBitrate:"bitrate",
			trackRating:"rating",
			trackPlaycount:"playcount",
			trackTracknum:"tracknum",
			trackSamplerate:"samplerate",
			trackUrl:"url",
			trackLyrics:"lyrics",
			trackYear:"year"
		},
		operate: [
			{
				'run': function(val) { return (val == null)?null:String(val).split(',').map(Number)}, 'on': 'trackArtistIds'},{
				'run': function(val) { return (val == null)?null:String(val).split(',').map(Number)}, 'on': 'trackGenreIds'},{
				'run': function(val) { return (val == null)?null:Number.parseInt( val)}, 'on': 'trackArtistId'},{
				'run': function(val) { return (val == null)?null:Number.parseInt( val)}, 'on': 'trackAlbumId'},{
				'run': function(val) { return Number.parseInt( val)}, 'on': 'trackFilesize'},{
				'run': function(val) { return Number.parseInt( val)}, 'on': 'trackModificationTime'},{
				'run': function(val) { return Number.parseInt( val)}, 'on': 'trackAddedTime'},{
				'run': function(val) { return (val == null)?null:Number.parseInt( val)}, 'on': 'trackGenreId'},{
				'run': function(val) { return (val == null)?null:Number.parseInt( val)}, 'on': 'trackRating'},{
				'run': function(val) { return (val == null)?null:Number.parseInt( val)}, 'on': 'trackPlaycount'},{
				'run': function(val) { return (val == null)?null:Number.parseInt( val)}, 'on': 'trackTracknum'},{
				'run': function(val) { return Number.parseInt( val)}, 'on': 'trackSamplerate'},{
				'run': function(val) { return (val == null)?null:Number.parseInt( val)}, 'on': 'trackYear'},{
				'run': function(val) { return (val == null)?null:Number.parseInt( val)}, 'on': 'trackCompilation'
			}
		]
	};

	this.trackAttributes = {
		trackTitle:{ text: "Title"},
		trackArtist:{ text: "Artist"},
		trackAlbum:{ text: "Album"},
		trackYear:{ text: "Year"},
		trackDuration:{ text: "Duration", funct: utils.secsToDuration},
		trackGenres:{ text: "Genre"},
		trackTracknum:{ text: "Track #"},
		trackDisc:{ text: "Disc #"},
		trackDisccount:{ text: "Discs"},
		trackCompilation:{ text: "Compilation"},
		trackComment:{ text: "Comment"},
		trackRating:{ text: "Rating", funct: utils.getPrettyRating},
		trackPlaycount:{ text: "Play count"},
		trackAddedTime:{ text: "Added", funct: utils.dateToString},
		trackModificationTime:{ text: "Modified", funct: utils.dateToString},
		trackBitrate:{ text: "Bitrate"},
		trackSamplerate:{ text: "Samplerate"},
		trackFilesize:{ text: "File size", funct: utils.bytesToSize},
		trackUrl:{ text: "URL", funct: decodeURI},
		trackLyrics:{ text: "Lyrics"},
/*
		trackId:{ text: "id"},
		trackCoverid:{ text: "coverid"},
		trackAlbumId:{ text: "album_id"},
		trackGenre:{ text: "genre"},
		trackArtworkTrackId:{ text: "artwork_track_id"},
		trackGenreId:{ text: "genre_id"},
		trackGenreIds:{ text: "genre_ids"},
		trackArtistId:{ text: "artist_id"}
*/
	};

	this.postRequest = function( req) {
		var request = ['-'];
		request.push( req);
		var p = JSON.stringify({ id: 1, method: "slim.request", params: request});
		console.log("postRequest ["+config.settings.apiUrl+"]["+request+"][ "+p+" ]");
		return fetchModule.fetch(config.settings.apiUrl, {
			method: "POST",
			headers: {
				"Content-Type": "application/json"
			},
			body: JSON.stringify({
						id: 1,
						method: "slim.request",
						params: request
					})
			})
			.then(this.handleErrors)
			.then(function(response) {
				return response.json();
			})
			.then(function(data) {
				//console.dump(data);
				return data.result;
			});
	}

	this.getPlaylist = function( plsId) {
		const postRequest = this.postRequest;
		const playlistTracksMap = this.playlistTracksMap;
		return new Promise(function (resolve, reject) {
			postRequest( ["playlists", "tracks", 0, config.settings.maxFetchItems, "playlist_id:"+plsId,"tags:adlu"])
				.catch(function(error) {
					console.log(error);
					reject( Error( error));
				})
				.then(function( result) {
					//console.dump( result);
					var dataTransform = DataTransform(result, playlistTracksMap);
					resolve( dataTransform.transform() );
				});
		});
	}

	this.getAllPlaylists = function( ) {
		postRequest = this.postRequest;
		playlistMap = this.playlistMap;
		return new Promise(function (resolve, reject) {
			postRequest( ["playlists", 0, config.settings.maxFetchItems])
				.catch(function(error) {
					console.log(error);
					reject( Error( error));
				})
				.then(function( result) {
					//console.dump( result);
					var dataTransform = DataTransform(result, playlistMap);
					resolve( dataTransform.transform() );
				});
		});
	}

	this.getAllArtists = function( ) {
		postRequest = this.postRequest;
		artistMap = this.artistMap;
		return new Promise(function (resolve, reject) {
			postRequest( ["artists", 0, config.settings.maxFetchItems])
				.catch(function(error) {
					console.log(error);
					reject( Error( error));
				})
				.then(function( result) {
					//console.dump( result);
					var dataTransform = DataTransform(result, artistMap);
					resolve( dataTransform.transform() );
				});
		});
	}

	this.getAllAlbums = function( ) {
		postRequest = this.postRequest;
		albumMap = this.albumMap;
		return new Promise(function (resolve, reject) {
			postRequest( ["albums", 0, config.settings.maxFetchItems,"tags:lyiwaSSaaS"])
				.catch(function(error) {
					console.log(error);
					reject( Error( error));
				})
				.then(function( result) {
					//console.dump( result);
					var dataTransform = DataTransform(result, albumMap);
					resolve( dataTransform.transform() );
				});
		});
	}

	this.getAllTracks = function( collTracks, progressInterface) {
		getTracks = this.getTracks;
		//var that = this;
		
		this.getTrackCount().then( numTracks => {
			collTracks.clear();
			numTracks = Math.min(numTracks, config.settings.maxFetchItems);
			progressInterface.setMaxValue(numTracks);

			console.log("worker init numTracks = "+numTracks);
			var currTrack = 0;
			const worker = new Worker('./lms-track-worker');
			worker.postMessage({ startAt: 0, fetchChunks: config.settings.fetchChunks });

			worker.onmessage = function( msg) {
				console.log("track worker onMsg currTrack="+currTrack+" tracksRead="+msg.data.tracksRead+" tracks.length="+msg.data.tracks.length);
				if ( msg.data.success) {
					//console.log( "config.settings.fetchChunks="+config.settings.fetchChunks);
					collTracks.insert( msg.data.tracks);
					currTrack += config.settings.fetchChunks;
					
					if( currTrack <= numTracks) {
						progressInterface.setCurrValue( currTrack);
						progressInterface.setMessage( currTrack+" of "+numTracks+" tracks imported");
						console.log("track worker onMsg new msg startAt="+currTrack);
						worker.postMessage({ startAt: currTrack, fetchChunks: config.settings.fetchChunks });
					} else {
						console.log("track worker done currTrack="+currTrack);
						progressInterface.setMessage( "All "+numTracks+" tracks imported");
						progressInterface.setCurrValue( numTracks);
						progressInterface.setBusy( false);
						worker.terminate();
					}
				} else {
					console.log("track worker NO success");
					worker.terminate();
				}
			};
			worker.onerror = function(err) {
				console.log(`An unhandled error occurred in worker: ${err.filename}, line: ${err.lineno} :`);
				console.log(err.message);
			};

		}, function(error) {
			progressInterface.setBusy( false);
		});
	}

	// used by worker
	this.getTracks = function( from, count ) {
		console.log("getTracks from="+from+" count="+count);
		postRequest = this.postRequest;
		trackMap = this.trackMap;
		return new Promise(function (resolve, reject) {
			postRequest( ["titles", from, count,"tags:acCdefgGiJklnpPDqrROstSTuwy"])
				.catch(function(error) {
					console.log(error);
					reject( Error( error));
				})
				.then(function( result) {
					//console.dump( result);
					//console.log("getTracks result="+result.length);
					const dataTransform = DataTransform(result, trackMap);
					resolve( dataTransform.transform() );
				});
				/*.then( result => {
					console.log("getTracks result2="+result.length);
					//collTracks.insert( result);
					resolve( result);
				});*/
		});
	}

	this.getTrackCount = function( ) {
		postRequest = this.postRequest;
		return new Promise(function (resolve, reject) {
			postRequest( ["info", "total", "songs", "?"])
				.catch(function(error) {
					console.log(error);
					reject( Error( error));
				})
				.then(function( result) {
					resolve( result._songs );
				});
		});
	}

	this.handleErrors = function(response) {
		if (!response.ok) {
			console.log(JSON.stringify(response));
			throw Error(response.statusText);
		}
		return response;
	}

	this.showLMSError = function( error) {
		this.feedback.error({
			title: "LMS Error",
			message: "LMS connection: "+config.settings.apiUrl+" \nError:"+error.message,
			duration: 10000
		});
	}

	this.checkConn = function() {
		//console.log("conn="+Connectivity.getConnectionType());
		if( config.settings.warnIfNoWifi && Connectivity.getConnectionType() != Connectivity.connectionType.wifi) {
			this.feedback.warning({
				title: "WiFi unavailable",
				message: "WiFi connection not available - Import unlikely to work"
			});
			return false;
		}
		return true;
	}
}

exports.LMS = LMS;